# Generated by roxygen2: do not edit by hand

S3method("!=",Container)
S3method("$<-",Container)
S3method("&",Dict)
S3method("&",Set)
S3method("+",Container)
S3method("+",Deque)
S3method("+",Dict)
S3method("+",Set)
S3method("-",Container)
S3method("-",Deque)
S3method("-",Dict)
S3method("-",Set)
S3method("<",Container)
S3method("<=",Container)
S3method("==",Container)
S3method(">",Container)
S3method(">=",Container)
S3method("[",Container)
S3method("[<-",Container)
S3method("[[",Container)
S3method("[[<-",Container)
S3method("names<-",Container)
S3method("|",Dict)
S3method("|",Set)
S3method(Math,Container)
S3method(Math,Deque)
S3method(Math,Dict)
S3method(Math,Set)
S3method(Summary,Container)
S3method(Summary,Deque)
S3method(Summary,Dict)
S3method(Summary,Set)
S3method(add,Container)
S3method(add,Dict)
S3method(add,dict.table)
S3method(addleft,Deque)
S3method(all,equal.Container)
S3method(as.dict.table,data.table)
S3method(as.dict.table,default)
S3method(as.list,Container)
S3method(at,Container)
S3method(at,dict.table)
S3method(at2,Container)
S3method(at2,dict.table)
S3method(c,Container)
S3method(c,Deque)
S3method(c,Dict)
S3method(c,OrderedSet)
S3method(c,Set)
S3method(cbind,dict.table)
S3method(clear,Container)
S3method(clear,dict.table)
S3method(clone,Container)
S3method(clone,dict.table)
S3method(count,Container)
S3method(count,Set)
S3method(delete,Container)
S3method(delete_at,Container)
S3method(delete_at,dict.table)
S3method(discard,Container)
S3method(discard_at,Container)
S3method(discard_at,dict.table)
S3method(empty,Container)
S3method(format,Container)
S3method(format,Deque)
S3method(format,Dict)
S3method(format,OrderedSet)
S3method(format,Set)
S3method(format,list)
S3method(has,Container)
S3method(has,dict.table)
S3method(has_name,Container)
S3method(has_name,dict.table)
S3method(is_empty,Container)
S3method(is_empty,dict.table)
S3method(iter,Container)
S3method(iter,default)
S3method(length,Container)
S3method(length,Iterator)
S3method(names,Container)
S3method(peek,Deque)
S3method(peek_at,Container)
S3method(peek_at,dict.table)
S3method(peek_at2,Container)
S3method(peek_at2,dict.table)
S3method(peekleft,Deque)
S3method(print,dict.table)
S3method(rbind,dict.table)
S3method(ref_add,Container)
S3method(ref_add,Dict)
S3method(ref_add,dict.table)
S3method(ref_addleft,Deque)
S3method(ref_clear,Container)
S3method(ref_clear,dict.table)
S3method(ref_delete,Container)
S3method(ref_delete_at,Container)
S3method(ref_delete_at,dict.table)
S3method(ref_discard,Container)
S3method(ref_discard_at,Container)
S3method(ref_discard_at,dict.table)
S3method(ref_pop,Container)
S3method(ref_pop,Deque)
S3method(ref_pop,dict.table)
S3method(ref_popleft,Deque)
S3method(ref_rename,Container)
S3method(ref_rename,dict.table)
S3method(ref_replace,Container)
S3method(ref_replace,Dict)
S3method(ref_replace_at,Container)
S3method(ref_replace_at,dict.table)
S3method(ref_rev,Deque)
S3method(ref_rotate,Deque)
S3method(ref_update,Container)
S3method(ref_update,dict.table)
S3method(rename,Container)
S3method(rename,default)
S3method(rename,dict.table)
S3method(replace,Container)
S3method(replace,Dict)
S3method(replace,default)
S3method(replace_at,Container)
S3method(replace_at,dict.table)
S3method(rev,Deque)
S3method(rotate,Deque)
S3method(size,Container)
S3method(sortkey,Dict)
S3method(str,Container)
S3method(update,Container)
S3method(update,dict.table)
S3method(update,list)
S3method(values,Container)
S3method(values,dict.table)
export("%in%")
export(Container)
export(Deque)
export(Dict)
export(Iterator)
export(OrderedSet)
export(Set)
export(add)
export(addleft)
export(as.cont)
export(as.container)
export(as.deque)
export(as.dict)
export(as.dict.table)
export(as.orderedset)
export(as.set)
export(at)
export(at2)
export(begin)
export(clear)
export(clone)
export(cont)
export(container)
export(container_options)
export(count)
export(delete)
export(delete_at)
export(deque)
export(dict)
export(dict.table)
export(discard)
export(discard_at)
export(empty)
export(getContainerOption)
export(get_next)
export(get_value)
export(has)
export(has_name)
export(has_next)
export(has_value)
export(is.container)
export(is.deque)
export(is.dict)
export(is.dict.table)
export(is.iterable)
export(is.iterator)
export(is.orderedset)
export(is.set)
export(is_empty)
export(iter)
export(keys)
export(next_iter)
export(peek)
export(peek_at)
export(peek_at2)
export(peekleft)
export(pos)
export(ref_add)
export(ref_addleft)
export(ref_clear)
export(ref_delete)
export(ref_delete_at)
export(ref_discard)
export(ref_discard_at)
export(ref_pop)
export(ref_popleft)
export(ref_rename)
export(ref_replace)
export(ref_replace_at)
export(ref_rev)
export(ref_rotate)
export(ref_update)
export(rename)
export(replace)
export(replace_at)
export(reset_iter)
export(rotate)
export(set)
export(setnew)
export(size)
export(sortkey)
export(unpack)
export(values)
exportMethods("%in%")
import(data.table)
import(methods)
importFrom(R6,R6Class)
